/***************************************************************************
                          daemon.cpp  -  description
                             -------------------
    begin                : Sat Sep 29 2001
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#pragma implementation

#include <iostream>

using namespace std;

#include <sys/time.h>
#include <sys/resource.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>

#include "daemon.h"

using namespace sucs;

struct sigaction ignoreChildDeath = { NULL, 0, SA_NOCLDSTOP | SA_RESTART, 0 };

void Daemon::Daemonise()
{
	struct rlimit resourceLimit = { 0 };
	int status = -1;
	int fileDesc = -1;

	// Make sure we don't zombify ourselves
	sigaction(SIGCHLD, &ignoreChildDeath, NULL);

	// Fork away from main process
	status = fork();
	switch(status)
	{
	case -1: return;
	case 0: break;
	default: exit(0);
	}

	// Change status to reflect daemon
	status = setsid();
	if(status == -1)
	{
		cerr << "Daemonise: PANIC: Problem setting status." << endl << flush;
		exit(1);
	}

	// Kill all file descriptors
	resourceLimit.rlim_max = 0;
	status = getrlimit(RLIMIT_NOFILE, &resourceLimit);
	if(status == -1)
	{
		cerr << "Daemonise: PANIC: Can't get resource limit." << endl << flush;
		exit(1);
	}
	if(resourceLimit.rlim_max == 0)
	{
		cerr << "Daemonise: PANIC: Silly maximum number of file descriptors." << endl << flush;
		exit(1);
	}
	for(int i = 0; i < resourceLimit.rlim_max; i++) close(i);

	// Fork again
	status = fork();
	switch(status)
	{
	case -1:
		exit(1);
	case 0:
		break;
	default:
		exit(0);
	}

    // Set up environment to reflect a daemon
	chdir("/");
	umask(0);
	fileDesc = open("/dev/null", O_RDWR);
	dup(fileDesc);
	dup(fileDesc);

    // Call the actual daemon code
	exit(RunProgram());
}

Daemon::Daemon()
{
}

Daemon::~Daemon()
{
}
