//
// C++ Implementation: condition
//
// Description:
//
//
// Author: Gav Wood <gav@indigoarchive.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __SUCS 1

#include <cassert>
using namespace std;

#include <errno.h>
#include <pthread.h>

#include "condition.h"

namespace sucs
{

Condition::Condition()
{
	theVar = new pthread_cond_t;
	pthread_cond_init(theVar, NULL);
	theMutex = 0;
}

Condition::Condition(const Mutex &mutex)
{
	theVar = new pthread_cond_t;
	pthread_cond_init(theVar, NULL);
	theMutex = &mutex;
}

Condition::~Condition()
{
	pthread_cond_destroy(theVar);
	delete theVar;
}

void Condition::signal(bool useMutex) const
{
	if(useMutex && theMutex) theMutex->lock();
	assert(!pthread_cond_signal(theVar));
	if(useMutex && theMutex) theMutex->unlock();
}

void Condition::broadcast(bool useMutex) const
{
	if(useMutex && theMutex) theMutex->lock();
	assert(!pthread_cond_broadcast(theVar));
	if(useMutex && theMutex) theMutex->unlock();
}

void Condition::signal(const Mutex &mutex) const
{
	mutex.lock();
	assert(!pthread_cond_signal(theVar));
	mutex.unlock();
}

void Condition::broadcast(const Mutex &mutex) const
{
	mutex.lock();
	assert(!pthread_cond_broadcast(theVar));
	mutex.unlock();
}

void Condition::setMutex(const Mutex &mutex)
{
	theMutex = &mutex;
}

void Condition::wait() const
{
	assert(theMutex);
	assert(!pthread_cond_wait(theVar, theMutex->theVar));
}

void Condition::wait(const Mutex &mutex) const
{
	assert(!pthread_cond_wait(theVar, mutex.theVar));
}

void Condition::lockAndWait() const
{
	assert(theMutex);
	theMutex->lock();
	assert(!pthread_cond_wait(theVar, theMutex->theVar));
}

void Condition::lockAndWait(const Mutex &mutex) const
{
	mutex.lock();
	assert(!pthread_cond_wait(theVar, mutex.theVar));
}

};
