//
// C++ Interface: mutex
//
// Description:
//
//
// Author: Gav Wood <gav@indigoarchive.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef _SUCS_MUTEX_H
#define _SUCS_MUTEX_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#pragma interface

#include <string>
using namespace std;

#include <pthread.h>

namespace sucs
{

/**
@author Gav Wood
*/

class Condition;

class Mutex
{
	friend class Condition;

	mutable pthread_mutex_t *theVar;
	
	string theName;

public:
	/** Lock the mutex. Returns true if the mutex was locked successfully.
	 * Returns false on error, i.e. the mutex is already locked under this thread.
	 */
	bool lock() const;
	
	/** Lock the mutex. Returns true if the mutex was locked successfully.
	 * Returns false if the mutex is already locked.
	 */
	bool tryLock() const;
	
	/** Unlock the mutex. Returns true if the mutex was unlocked successfully.
	 * Returns false on error, i.e. the mutex is locked under another thread.
	 */
	bool unlock() const;

	Mutex(string name = "");
	~Mutex();
};

};

#endif
