/***************************************************************************
                          semaphore.cpp  -  description
                             -------------------
    begin                : Sun Jan 9 2000
    copyright            : (C) 2000 by Gavin James Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __SUCS 1

#pragma implementation

#include <iostream>
using namespace std;

#include <semaphore.h>

#include "sema.h"
using namespace sucs;

int Sema::numSemas = 0;

const int Sema::semaValue() const
{
	int i;
	sem_getvalue(&theSem, &i);
	return i;
}

Sema::operator int() const
{
	return semaValue();
}

void Sema::hop() const
{
	wait();
	signal();
}

void Sema::engage() const
{
	while(tryWait());
}

bool Sema::tryWait() const
{
	return sem_trywait(&theSem) == 0;
}

bool Sema::wait(bool dontReturnUntilTrue) const
{
	bool ret = false;
	for(ret = !sem_wait(&theSem); dontReturnUntilTrue && !ret; ret = !sem_wait(&theSem));
	return ret;
}

const Sema &Sema::operator--() const
{
	wait();
	return *this;
}

void Sema::signal() const
{
	if(semaValue() < theNumEntries)
		sem_post(&theSem);
//	else
//		cerr << this << ": Signalling error; attempt to signal past entries." << endl << flush;
}
	
const Sema &Sema::operator++() const
{
	signal();
	return *this;
}

void Sema::init(int numEntries)
{
	sem_destroy(&theSem);
	sem_init(&theSem, 0, numEntries);
	theNumEntries = numEntries;
}

Sema::Sema(int numEntries)
{
//	cerr << "Creating semaphore [" << ++numSemas << " in existance]" << endl;
//	cerr << this << ": Initialising semaphore with " << numEntries << " entries" << endl;
	if(sem_init(&theSem, 0, numEntries))
		cerr << this << ": Error initialising semaphore: " << strerror(errno) << endl << flush;
//	cerr << this << ": Waits left: " << semaValue() << endl;
	theNumEntries = numEntries;
}

Sema::Sema(const Sema &newSema)
{
//	cerr << "Creating semaphore [" << ++numSemas << " in existance]" << endl;
	sem_init(&theSem, 0, newSema.theNumEntries);
	theNumEntries = newSema.theNumEntries;
}

Sema::~Sema()
{
//	cerr << "Deleting semaphore [" << --numSemas << " in existance]" << endl;
	sem_destroy(&theSem);
}
