/***************************************************************************
                          kohonendata.cpp  -  description
                             -------------------
    begin                : Thu Jan 6 2000
    copyright            : (C) 2000 by Gavin James Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#pragma implementation

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <cmath>

using namespace std;

#include "kohonennet.h"

#include "kohonendata.h"

using namespace sucs;

string KohonenData::getClass(unsigned index, const KohonenNet &theNet) const
{
	classifierNet = &theNet;
	return theNet.Classify(theValues[index]);
}

string KohonenData::getClass(unsigned index) const
{
	return classifierNet->Classify(theValues[index]);
}

void KohonenData::setClassifier(const KohonenNet &theNet) const
{
	classifierNet = &theNet;
}

void KohonenData::readDat(istream &fin)
{
	unsigned Dims, Lines = 0;
	fin >> Dims;
	int pos = fin.tellg();
	while(!fin.eof()) if(fin.get() == '\n') Lines++;
	Lines--;
	Deallocate();
	Allocate(Dims, Lines);

	fin.clear();
	fin.seekg(pos);
	char Buffer[1024];
	for(unsigned i = 0; !fin.eof() && i < Lines; i++)
	{	for(unsigned d = 0; d < Dims; d++)
			fin >> theValues[i][d];
		while(fin.peek() == '\t' || fin.peek() == ' ') fin.get();
		fin.getline(Buffer, 1024);
		theLabels[i] = Buffer;
	}
}

void KohonenData::setLabel(unsigned index, const string &label)
{
#ifdef DEBUG
	if(index >= theCount) exit(1);
#endif
	theLabels[index] = label;
}

void KohonenData::setValue(unsigned index, unsigned dimension, double value)
{
#ifdef DEBUG
	if(index >= theCount) exit(1);
	if(dimension >= theDimensions) exit(1);
#endif
	theValues[index][dimension] = value;
}

void KohonenData::Allocate(unsigned Dimensions, unsigned Count)
{
	theValues = new double *[Count];
	theLabels = new string[Count];
	for(unsigned i = 0; i < Count; i++)
		theValues[i] = new double[Dimensions];
	theCount = Count;
	theDimensions = Dimensions;
}

void KohonenData::Deallocate()
{
	if(theValues || theLabels)
	{
		if(!(theValues && theLabels)) exit(1);
		for(unsigned i = 0; i < theCount; i++)
			delete [] theValues[i];
		delete [] theValues;
		delete [] theLabels;
	}
	theDimensions = 0;
	theCount = 0;
	theValues = 0;
	theLabels = 0;
}

void KohonenData::Init(unsigned Dimensions, unsigned Count)
{
	Deallocate();
	Allocate(Dimensions, Count);
	for(unsigned i = 0; i < theCount; i++)
	{	for(unsigned d = 0; d < theDimensions; d++)
			theValues[i][d] = 0;
		theLabels[i] = "";
	}
}

void KohonenData::catenate(const KohonenData &src)
{
	if(src.theDimensions != theDimensions)
	{	cerr << "Dimension mismatch! (" << src.theDimensions << " != " << theDimensions << ")" << endl;
		return;
	}

	unsigned newCount = src.theCount + theCount;
	double **newValues = new double *[newCount];
	for(unsigned i = 0; i < newCount; i++)
		newValues[i] = new double[theDimensions];
	string *newLabels = new string[newCount];

	unsigned i = 0;
	for(; i < theCount; i++)
	{	for(unsigned j = 0; j < theDimensions; j++)
			newValues[i][j] = theValues[i][j];
		newLabels[i] = theLabels[i];
	}
	for(; i < newCount; i++)
	{	for(unsigned j = 0; j < theDimensions; j++)
			newValues[i][j] = src.theValues[i - theCount][j];
		newLabels[i] = src.theLabels[i - theCount];
	}

	Deallocate();
	theValues = newValues;
	theLabels = newLabels;
	theCount = newCount;
	theDimensions = src.theDimensions;
}

KohonenData::KohonenData()
{
	theValues = 0;
	theLabels = 0;
	theDimensions = 0;
	theCount = 0;
}

KohonenData::~KohonenData()
{
	Deallocate();
}
