/***************************************************************************
                          kohonennet.h  -  description
                             -------------------
    begin                : Sun Jan 9 2000
    copyright            : (C) 2000 by Gavin James Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifndef __KOHONENNET
#define __KOHENENNET

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#pragma interface

#include <iostream>
#include <fstream>
#include <string>

using namespace std;

namespace sucs
{

class KohonenData;

class KohonenNet
{
	unsigned theWidth, theHeight, theDimensions;
	double ***theValues;
	string **theLabels;

	void update(const double *Vector, double Alpha, double Diameter);
	double closest(const double *Vector, unsigned &returnX, unsigned &returnY) const;
	virtual double proximity(unsigned x1, unsigned y1, unsigned x2, unsigned y2, double Diameter) const;

protected:
	void Allocate(unsigned Width, unsigned Height, unsigned Dimensions);
	void Deallocate();

public:
	void Init(unsigned Width, unsigned Height, unsigned Dimensions, double Low, double High);
	bool Train(unsigned RunLength, double Alpha, double Diameter, double LambdaA, double LambdaD, const KohonenData &Data);
	void Label(double High, double Low, const KohonenData &Data);
	double Classify(const double *Vector, unsigned &returnX, unsigned &returnY) { return closest(Vector, returnX, returnY); }
	string Classify(const double *Vector) const;

	void setLabel(unsigned X, unsigned Y, const string &newLabel) { theLabels[X][Y] = newLabel; }
	string getLabel(unsigned X, unsigned Y) { return theLabels[X][Y]; }
	void setValue(unsigned X, unsigned Y, unsigned D, double newValue) { theValues[X][Y][D] = newValue; }
	double getValue(unsigned X, unsigned Y, unsigned D) { return theValues[X][Y][D]; }
	void setValues(unsigned X, unsigned Y, double *newValues) { for(unsigned i = 0; i < theDimensions; i++) theValues[X][Y][i] = newValues[i]; }
	const double *getValues(unsigned X, unsigned Y) { return theValues[X][Y]; }

	unsigned getWidth() { return theWidth; }
	unsigned getHeight() { return theHeight; }
	unsigned getDimensions() { return theDimensions; }

	void readCod(istream &fin);
	void writeCod(ostream &fout);

	KohonenNet();
	virtual ~KohonenNet();

	// OBSOLETE
	bool Train(unsigned RunLength, double Alpha, double Diameter, double K, const KohonenData &Data);	
	bool DoTraining(unsigned RunLength, double Alpha, double Diameter, double Lambda, const KohonenData &Data);
};

}

#endif
