//
// C++ Implementation: referencecounted
//
// Description:
//
//
// Author: Gav Wood <gav@indigoarchive.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <cassert>
using namespace std;

#include "sema.h"
#include "referencecounted.h"

namespace sucs
{

unsigned int ReferenceCounted::nextRef = 1;
Sema ReferenceCounted::theLock;
map<unsigned int, unsigned int> ReferenceCounted::theRefs;

ReferenceCounted::ReferenceCounted()
{
	refId = nextRef;
	nextRef++;
	assert(nextRef);
	
	theLock.wait();
	theRefs[refId] = 1;
	theLock.signal();
}

ReferenceCounted::ReferenceCounted(const ReferenceCounted &copy)
{
	refId = copy.refId;
	theLock.wait();
	theRefs[refId]++;
	theLock.signal();
}
	
ReferenceCounted::~ReferenceCounted()
{
	theLock.wait();
	theRefs[refId]--;
	theLock.signal();
}

const bool ReferenceCounted::lastInstance()
{
	theLock.wait();
	bool ret = theRefs[refId] == 1;
	theLock.signal();
	return ret;
}

ReferenceCounted &ReferenceCounted::operator=(const ReferenceCounted &source)
{
	refId = source.refId;
	theLock.wait();
	theRefs[refId]++;
	theLock.signal();
	return *this;
}

const unsigned int ReferenceCounted::reference() const
{
	return refId;
}

};
