/***************************************************************************
                          mast.cpp  -  The main mast implementation file
                             -------------------
    begin                : Tue 20 November 2001
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __SUCS 1
#pragma implementation

#include <iostream>
#include <string>
//#include <cstring>
#include <cassert>
using namespace std;

#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <netdb.h>
#include <unistd.h>
#include <fcntl.h>

#include "link.h"
#include "mast.h"
#include "thread.h"
using namespace sucs;

namespace sucs
{

Link::Link(): Socket()
{
	theMast = 0;
}

Link::Link(int socket): Socket(socket)
{
	adopt(socket);
}

Link::Link(const Link &src) : Socket(src)
{
	theMast = src.theMast;
}

void Link::adopt(int socket)
{
	theSocket = socket;
	init();
}

void Link::waitForInput() const
{
	assert(theMast);
	setBlocking(false);
	while(!poll())
	{	setBlocking(true);
		theMast->theGotIO.wait();
		setBlocking(false);
	}
}

const bool Link::send(const string &theString) const
{
	return send(theString.c_str(), theString.length());
}

const bool Link::send(const char *Buffer, int Len) const
{
	int sent = 0;
	for(int last = 0; sent < Len && last >= 0; sent += last)
		last = ::send(theSocket, Buffer, Len, 0);
	return sent == Len;
}

const bool Link::send(const unsigned char Data) const
{
	return ::send(theSocket, &Data, 1, 0) == 1;
}

const string Link::name() const
{
	char *buffer = new char[80];
	int len = 80;
	if(getsockname(theSocket, (struct sockaddr *)buffer, (socklen_t *)&len) == -1)
		return "";
	else
	{	buffer[len] = 0;
		return buffer;
	}
	delete [] buffer;
}

const int Link::receive(char *Buffer, int Len, int timeOut, int *timeLeft)
{
	int r;
	if(timeOut)
	{
		int t;
		for(t = 0, r = recv(theSocket, Buffer, Len, MSG_DONTWAIT); (r == 0 || (r < 0 && errno == EAGAIN)) && t < timeOut; t+=10000)
		{	usleep(10000);
			r = recv(theSocket, Buffer, Len, MSG_DONTWAIT);
		}
		if(timeLeft) *timeLeft = timeOut - t;
	}
	else
		r = recv(theSocket, Buffer, Len, 0);
	
	if(r <= 0)
	{
		terr("*** ERROR: Socket receive error " << r << " (" << errno << "): " << strerror(errno));
		cleanup();
		return 0;
	}
	return r;
}

void Link::receiveChunk(char *Buffer, int Len, int timeOut)
{
	if(timeOut)
		for(int got = 0, t = timeOut; got < Len && t > 0 && isOpen(); got += receive(Buffer + got, Len - got, t, &t));
	else
		for(int got = 0; got < Len && isOpen(); got += receive(Buffer + got, Len - got));
}

const string Link::receiveLine()
{
	string ret = "";
	char c = 0; 
	while(receive(&c, 1) && isOpen())
		if(c == '\n') break;
		else if(c != 13 && c != 10 && c != 0) ret += c;
	return ret;
}

const string Link::receiveUntil(char *until)
{
	string ret = "";
	char c = 0; 
	while(receive(&c, 1) && isOpen())
		if(string(until).find(c) != string::npos) break;
		else if(!(string(until).find("\n") != string::npos && (c == 13 || c == 10)) && c != 0) ret += c;
	return ret;
}

void Link::disconnect()
{
	cleanup();
}

void Link::cleanup()
{
	wait();
	if(isOpen())
	{	deinit();
		close();
	}
	signal();
}

const ConditionMutex *Link::gotIO()
{
	return &(theMast->theGotIO);
}

const bool Link::connect(const string anAddress, int aPort)
{
	if(theSocket != -1)
		return false;
	theSocket = socket(PF_INET, SOCK_STREAM, 0);

	struct hostent *h = gethostbyname(anAddress.c_str());
	if(!h)
	{	close();
		return false;
	}
	struct sockaddr_in theAddr;
	theAddr.sin_family = AF_INET;
	theAddr.sin_port = htons(aPort);
	theAddr.sin_addr = *(struct in_addr *)(h->h_addr);
	bzero(&(theAddr.sin_zero), 8);
	if(::connect(theSocket, (struct sockaddr *)&theAddr, sizeof(struct sockaddr)))
	{	close();
		return false;
	}
	
	init();
	return true;
}

}
