//
// C++ Implementation: socket
//
// Description:
//
//
// Author: Gav Wood <gav@indigoarchive.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __SUCS 1

#include <cassert>
using namespace std;

#include <sys/socket.h>
#include <sys/poll.h>
#include <unistd.h>
#include <fcntl.h>

#include "socket.h"

namespace sucs
{

map<int, int> Socket::refCount;

Socket::Socket()
{
	theSocket = -1;
}

Socket::Socket(int socket)
{
	theSocket = socket;
}

Socket::~Socket()
{
	if(theSocket > -1)
		if(refCount[theSocket] > 0)
			refCount[theSocket]--;
		else
			cleanup();
}

Socket::Socket(const Socket &src)
{
	if(src.theSocket > -1)
	{	refCount[src.theSocket]++;
		theSocket = src.theSocket;
	}
}

Socket &Socket::operator=(const Socket &src)
{
	if(theSocket > -1)
		if(refCount[theSocket] > 0)
			refCount[theSocket]--;
		else
			cleanup();
	theSocket = src.theSocket;
	if(theSocket > -1)
		refCount[theSocket]++;
	return *this;
}

void Socket::setBlocking(bool blocking) const
{
	fcntl(theSocket, F_SETFL, blocking ? O_ASYNC : O_NONBLOCK);
}

const bool Socket::poll() const
{
	struct pollfd *p = new struct pollfd;
	p->fd = theSocket;
	p->events = POLLIN | POLLPRI | POLLHUP | POLLERR;
	int ret;
	assert((ret = ::poll(p, 1, 0)) != -1);
	return ret;
	
/*	fd_set rfds;
	struct timeval tv;
	tv.tv_sec = 0;
	tv.tv_usec = 1;
	FD_ZERO(&rfds);
	FD_SET(theSocket, &rfds);

	return select(theSocket + 1, &rfds, NULL, NULL, &tv);
*/
}

void Socket::cleanup()
{
	close();
}

void Socket::close()
{
	::close(theSocket);
	theSocket = -1;
}

};
