/***************************************************************************
                          stub.cpp  -  Simple network server.
                             -------------------
    begin                : Tue Sep 18 2001
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __SUCS 1

#include <iostream>
#include <string>
#include <list>

using namespace std;

#include "mast.h"
#include "link.h"
#include "threaded.h"
#include "threaded.h"

using namespace sucs;

class TestLink: public Link, protected Threaded
{
	MemberThread theThread;
	
	virtual void threadCleanup(MemberThread *) { tdbg("TestLink::threadCleanup()" << endl); if(mast()) { cleanupWait(); /*mast()->safeDelete(this);*/ } }
	// THIS IS BAD PRACTISE, and can cause memory errors, since ghost will get
	// called before the cleanup handler is out. A better solution would be to have mast
	// watch the thread and make sure it is exitted before it's deleted.
	virtual void threadBody(MemberThread *)
	{	if(mast())
		{	while(isOpen())
			{	waitForInput();
				cout << "\r[remote] " << receiveLine() << endl << "> " << flush;
			}
		}
		else
		{
			char b[80];
			for(int r = receive(b, 79); r; r = receive(b, 79))
			{	b[r] = 0;
				cout << "\r[remote-" << r << "] " << b << endl << "> " << flush;
				if(b[0] == 'q' && r == 1) { close(); break; }
			}
		}
	}

protected:
	virtual void init() { theThread.start(); }
	virtual void deinit() { theThread.cancelAndWait(); }
	
public:
	TestLink() : Link(), theThread(this, "TestLink") {}
	~TestLink() {}
};


class TestMast: public Mast
{
protected:
	virtual Link *newArriver() { return new TestLink; }
	
public:
	void sendAll(const string &s)
	{	for(list<Link *>::iterator j = theLinks.begin(); j != theLinks.end(); j++)
			(*j)->send(s);
	}
};

int main()
{
	TestLink myLink;
	char s[80];
	if(myLink.connect("localhost", 16661))
	{
		while((s[0] != 'q' || s[1]) && myLink.isOpen())
		{	cout << "> " << flush;
			cin.getline(s, 80);
			string sn = s;
			myLink.send(sn);
		}
		if(myLink.isOpen())
			myLink.close();
	}
	else
	{
		TestMast *myMast = new TestMast;
		if(!myMast->start(16661))
		{	cout << "Problem starting the mast." << endl;
			return 1;
		}
		cout << "Listening for links..." << endl;
		
		while(s[0] != 'q' || s[1])
		{	cout << "> " << flush;
			cin.getline(s, 80);
			string sn = s;
			myMast->sendAll(sn);
		}
		myMast->quit();
		myMast->handleQuit();
		delete myMast;
	}
}
