/***************************************************************************
                          ssmatrix.h  -  Main Self-Similarity Matrix header
                                           file.
                             -------------------
    begin                : Tue 20 November 2001
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifndef __SSMATRIX
#define __SSMATRIX

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#pragma interface

#include <vector>
using namespace std;

#ifdef __SUCS
#include "spectrum.h"
#else
#include <sucs/spectrum.h>
#endif
using namespace sucs;

namespace sucs
{

class BeatSpectrum: public InverseSpectrum
{
public:
	BeatSpectrum(float *d, int size, float base = 0, bool logPowers = false) : InverseSpectrum(d, size, base, logPowers) {}
	BeatSpectrum(float base = 0, bool logPowers = false) : InverseSpectrum(base, logPowers) {}
};

class SSMatrix
{
	int numPoints;
	float theGapTime;
	vector< vector<float> > theMatrix;
	vector<Spectrum> theData;
	BeatSpectrum theBeatSpectrum;
	bool upToDate;
	float theOffset, theMagnitude;

protected:
	void updateBeatSpectrum();

public:
	void newPoint(const Spectrum &theSpectrum);
	void newPopulation(vector<Spectrum *> theSpectra);
	bool isFull() { return numPoints >= theData.size(); }
	void clear();

	const BeatSpectrum &beatSpectrum() { if(!upToDate) updateBeatSpectrum(); return theBeatSpectrum; }

	float matrixData(unsigned x, unsigned y) { return theMatrix[x][y]; }
	float matrixData(unsigned x, unsigned y, bool normalised) { return normalised ? (theMatrix[x][y] - theOffset) / theMagnitude : theMatrix[x][y]; }
	unsigned size() { return theMatrix.size(); }

	void resize(unsigned newSize);

	SSMatrix(int newSize, float gapTime = 0);
	~SSMatrix();
};

}

#endif
