/***************************************************************************
                          stub.cpp  -  Simple FFT example.
                             -------------------
    begin                : Tue Sep 18 2001
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __SUCS 1

#include <iostream>
#include <cmath>

using namespace std;

#include "fft.h"
#include "buffer.h"
#include "spectrum.h"

using namespace sucs;

int main(int argc, char **argv)
{
	Buffer myBuffer(22050.0, 4096.1/22050.0);
	FFT<float> myFFT(myBuffer);
	float theFrequency = argc > 1 ? atof(argv[1]) : 0;
	float theMagnitude = argc > 2 ? atof(argv[2]) : 1;
	
	while(theFrequency < 1 || theFrequency > 11025 || theMagnitude < 0 || theMagnitude > 1)
	{
		cout << "Enter frequency between 1Hz and 11025Hz: " << flush;
		cin >> theFrequency;
		cout << "Enter magnitude between 0 and 1: " << flush;
		cin >> theMagnitude;
	}
	
	myBuffer.clear();
	myBuffer.addPureTone(theFrequency, theMagnitude);
	myBuffer.applyHanning();
	FreeSpectrum mySpectrum = myFFT.perform(myBuffer);
/*	for(int i = 0; i < mySpectrum.size(); i+=1000)
		cout << "f = " << mySpectrum.bandFrequency(i) << "Hz: A = " << mySpectrum[i] << "dB (re 20Pa)" << endl;
	cout << "Maximum frequency = " << mySpectrum.getTopFrequency() << "Hz at Volume = " << mySpectrum[mySpectrum.getTopBand()] << "db (re 20Pa)" << endl;
*/	
	cout << "Maximum frequency = " << mySpectrum.getTopFrequency() << "Hz at Volume = K" << mySpectrum[mySpectrum.getTopBand()] << "Pa" << endl;
	BarkTemplate myBark;
	myBark.concrete(mySpectrum);
	mySpectrum.templateRescale(myBark);
	cout << "Maximum frequency = " << mySpectrum.getTopFrequency() << "Hz at Volume = K" << mySpectrum[mySpectrum.getTopBand()] << "Pa" << endl;
	
/*	for(int i = 0; i < mySpectrum.size(); i++)
//	for(int i = 0; i < mySpectrum.size(); i+=1000)
		cout << "f = " << mySpectrum.bandFrequency(i) << "Hz: A = " << mySpectrum[i] << "dB (re 20Pa)" << endl;
	cout << "Maximum frequency = " << mySpectrum.getTopFrequency() << "Hz at Volume = " << mySpectrum[mySpectrum.getTopBand()] << "db (re 20Pa)" << endl;
*/	
	mySpectrum.setMaxSPL(63.245553);		// ~64dB, re 20Pa
	mySpectrum.convertToDecibels(0.00002, false);
	cout << "Maximum frequency = " << mySpectrum.getTopFrequency() << "Hz at Volume = " << mySpectrum[mySpectrum.getTopBand()] << "dB" << endl;
	mySpectrum.convertToPhons();
	cout << "Maximum frequency = " << mySpectrum.getTopFrequency() << "Hz at Volume = " << mySpectrum[mySpectrum.getTopBand()] << "phons" << endl;
	mySpectrum.convertToSones();
	
/*	for(int i = 0; i < mySpectrum.size(); i++)
//	for(int i = 0; i < mySpectrum.size(); i+=1000)
		cout << "f = " << mySpectrum.bandFrequency(i) << "Hz: A = " << mySpectrum[i] << "sones" << endl;
*/
	cout << "Maximum frequency = " << mySpectrum.getTopFrequency() << "Hz at Volume = " << mySpectrum[mySpectrum.getTopBand()] << "sones" << endl;

}

