/***************************************************************************
                          thread.cpp  -  description
                             -------------------
    begin                : Thu Jan 6 2000
    copyright            : (C) 2000 by Gavin James Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#define DEBUG_MESSAGES 1
#define __SUCS 1
#pragma implementation

#include <iostream>
#include <string>
#include <cassert>
using namespace std;

#include <csignal>
#include <pthread.h>
#include <sys/time.h>

#include "thread.h"
using namespace sucs;

Sema sucs::__output;

int Thread::theIdCount = 0, Thread::theCount = 0, Thread::theNowRunningCount = 0;
Sema Thread::theNowRunning;
pthread_t Thread::theMainId = (pthread_t)NULL;
map<pthread_t, Thread *> Thread::theThreads;

ostream &sucs::operator<<(ostream &out, const Thread &t)
{
	if(&t)
		return out << t.id() << " (" << t.name() << "-" << t.threadId() << ")";
	else
		return out << "0 (main-" << Thread::theMainId << ")";
}

void Thread::waitForDeath() const
{
	theStartingUp.hop();
	theDead.hop();
}

bool Thread::cancel() const
{
	bool ret = false;
	theStartingUp.wait();
	if(theRunning)
	{	assert(!pthread_cancel(theThreadId));
		ret = true;
	}
	theStartingUp.signal();
	return ret;
}

bool Thread::cancelAndWait() const
{
	if(!cancel()) return false;
	waitForDeath();
	return true;
}

bool Thread::safelyStart(bool joinable, const unsigned int stackSize)
{
	if(isRunning())
		if(!cancel())
			return false;
		else
			waitForDeath();
	return start(stackSize, joinable);
}

bool Thread::start(bool joinable, const unsigned int stackSize)
{
	if(theStartingUp.tryWait())
	{	if(!theRunning)
		{	theJoinable = joinable;
			pthread_attr_t a;
			//TODO: Make this a bit more reliable...
			assert(!pthread_attr_init(&a));
			assert(!pthread_attr_setstacksize(&a, stackSize));
			assert(!pthread_attr_setdetachstate(&a, theJoinable ? PTHREAD_CREATE_JOINABLE : PTHREAD_CREATE_DETACHED));
			pthread_t t;
			int i = pthread_create(&t, &a, gameOn, (void *)this);
			assert(!pthread_attr_destroy(&a));
			if(i) terr("*** CRITICAL: Couldn't create new thread (" << errno << "): " << strerror(errno));
			return !i;
		}
		else
			theStartingUp.signal();
	}
	terr("*** WARNING: Can't start new thread " << (*this) << " - already one running!" << endl <<
	     "             Fix this with safelyStart() or better design!");
	return false;
}

void Thread::setMyImmediateCancel(bool immediate)
{
	assert(!pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL));
	assert(!pthread_setcanceltype(immediate ? PTHREAD_CANCEL_ASYNCHRONOUS : PTHREAD_CANCEL_DEFERRED, NULL));
	pthread_testcancel();
}

void *sucs::gameOn(void *arg)
{
	Thread::theNowRunning.wait();
	Thread::theNowRunning.signal();
	Thread *W = static_cast<Thread *>(arg);
	W->theThreadId = pthread_self();
	Thread::theThreads[W->theThreadId] = W;
	
	W->theDead.engage();
	assert(!pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL));
	assert(!pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL));

	W->theRunning = true;
	pthread_cleanup_push(cleanup, W);
	W->theStartingUp.signal();
	W->body();
	pthread_cleanup_pop(1);
	return NULL;
}

void sucs::cleanup(void *arg)
{
	Thread::theNowRunning.wait();
	Thread::theNowRunning.signal();
	Thread *W = static_cast<Thread *>(arg);
	
	W->theRunning = false;
	W->ghost();
	W->theDead.signal();
	Thread::theThreads.erase(pthread_self());
}

void *Thread::join() const
{
	void *ret = 0;
	theStartingUp.wait();
	theJoining.lock();
	if(theJoinable)
	{	theJoinable = false;
		assert(!pthread_join(theThreadId, &ret));
	}
	else
		terr("*** WARNING: Trying to join a thread that has been detached or already joined.");
	theJoining.unlock();
	theStartingUp.signal();
	return ret;
}

bool Thread::isRunning() const
{
	bool ret;
	theStartingUp.wait();
	ret = theRunning;
	theStartingUp.signal();
	return ret;
}

Thread::Thread(const string &name)
{
	if(!theCount)
		theMainId = pthread_self();
	theId = ++theIdCount;
	theName = name;
	theCount++;
	theRunning = false;
}

Thread::~Thread()
{
	theCount--;
	if(isRunning())
	{	terr("*** WARNING: Thread " << (*this) << " is still running in destructor. FIX THIS NOW.");
		cancel();
		waitForDeath();
	}
}

